#
# Copyright (c) 2025 CESNET z.s.p.o.
#
# This file is a part of pytest-oarepo (see https://github.com/oarepo/pytest-oarepo).
#
# pytest-oarepo is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
#
"""Base module for pytest-oarepo.

This module provides testing utilities and fixtures in OARepo applications.
"""

from __future__ import annotations

__version__ = "2.0.2"

__all__ = [
    "__version__",
]
