#
# Copyright (c) 2025 CESNET z.s.p.o.
#
# This file is a part of pytest-oarepo (see https://github.com/oarepo/pytest_oarepo).
#
# pytest-oarepo is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
#
"""Vocabularies module for pytest-oarepo.

This module provides testing utilities and fixtures for working with vocabularies
in OARepo applications.
"""

from __future__ import annotations

"""
from pytest_oarepo.vocabularies.loading import test_vocabularies

__all__ = ("test_vocabularies",)
"""
